﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data;
using System.Data.Common;
using System.Configuration;
using System.IO;
using System.Data.SqlClient;

namespace CS {
	class Program {
		static void Main(string[] args) {
			//EnumerateProviders();
			//SelectDbProvider("SqlNorthwind");
			//SelectDbProvider("MdbNorthwind");
			//ExecuteReader();
			//ExecuteScalar();
			//ExecuteNonQuery();
			ExecuteDataset();
			Console.ReadLine();
		}

		private static void ExecuteDataset() {
			DataSet ds = new DataSet();

			// Creazione della connessione
			using (SqlConnection conn = new SqlConnection(ConfigurationManager.ConnectionStrings["SqlNorthwind"].ConnectionString)) {
				// Apertura della connessione
				conn.Open();

				// Creazione del comando
				using (SqlCommand selectCmd = new SqlCommand("SELECT * FROM Customers;SELECT * FROM Suppliers;", conn)) {
					// Creazione dell'adapter
					using (SqlDataAdapter adapter = new SqlDataAdapter(selectCmd)) {
						// Popolamento dell'oggetto DataSet contenente due oggetti DataTable
						adapter.Fill(ds);

					} // L'adapter viene chiuso automaticamente alla fine del blocco Using

				} // Il comando viene chiuso automaticamente alla fine del blocco Using

			} // La connessione viene chiusa automaticamente alla fine del blocco Using

			// Il primo oggetto DataTable si riferisce ai clienti
			foreach (DataRow item in ds.Tables[0].Rows) {
				Console.WriteLine(item[0]);
			}

			// Il secondo oggetto DataTable si riferisce ai fornitori
			foreach (DataRow item in ds.Tables[1].Rows) {
				Console.WriteLine(item[0]);
			}
		}

		private static void EnumerateProviders() {
			DataTable providers = DbProviderFactories.GetFactoryClasses();
			foreach (DataRow provider in providers.Rows) {
				Console.WriteLine(provider[0]);
			}
		}

		private static void SelectDbProvider(string connStringName) {
			DataTable dt = new DataTable();

			// Nome invariante del managed data provider
			var providerInvariantName = ConfigurationManager.ConnectionStrings[connStringName].ProviderName;

			// Stringa di connessione
			var connectionString = ConfigurationManager.ConnectionStrings[connStringName].ConnectionString;

			// MS Access
			if (connStringName.Equals("MdbNorthwind")) {
				connectionString = GetFullPathNameFromDBFileName(connectionString);
				connectionString = "Provider=Microsoft.Jet.OLEDB.4.0; Data Source=" + connectionString;
			}

			// Creazione dell'oggetto factory
			DbProviderFactory factory = DbProviderFactories.GetFactory(providerInvariantName);

			// Interrogazione del database
			using (DbConnection conn = factory.CreateConnection()) {
				conn.ConnectionString = connectionString;
				conn.Open();

				// Modalità disconnessa: popolamento di un oggetto DataTable
				using (DbDataAdapter adatpter = factory.CreateDataAdapter()) {
					DbCommand selectCmd = factory.CreateCommand();
					selectCmd.CommandText = "SELECT * FROM Customers";
					selectCmd.Connection = conn;
					adatpter.SelectCommand = selectCmd;
					adatpter.Fill(dt);
				} // L'adapter viene chiuso automaticamente alla fine del blocco Using

			} // La connessione viene chiusa automaticamente alla fine del blocco Using

			Console.WriteLine(providerInvariantName);
			Console.WriteLine(connectionString);

			// Data binding della griglia di dati
			foreach (DataRow item in dt.Rows) {
				Console.WriteLine(item[0] + "-" + item[1]);
			}
		}

		// Ricava il percorso assoluto dal percorso relativo del file MDB
		private static string GetFullPathNameFromDBFileName(string relativeFileName) {
			relativeFileName = relativeFileName.Replace('/', '\\');

			if (relativeFileName.StartsWith("~\\"))
				relativeFileName = relativeFileName.Substring(2);
			else if (relativeFileName.StartsWith("\\"))
				relativeFileName = relativeFileName.Substring(1);

			return Path.Combine(AppDomain.CurrentDomain.BaseDirectory, relativeFileName);
		}

		private static void ExecuteReader() {
			StringBuilder builder = new StringBuilder();

			// Creazione della connessione
			using (SqlConnection conn = new SqlConnection(ConfigurationManager.ConnectionStrings["SqlNorthwind"].ConnectionString)) {
				// Apertura della connessione
				conn.Open();

				// Creazione del comando
				using (SqlCommand cmd = new SqlCommand("SELECT CustomerID, CompanyName, ContactName, City FROM Customers", conn)) {
					// Ritorna un resultset
					using (SqlDataReader reader = cmd.ExecuteReader()) {
						// Ciclo sul risultato della query
						while (reader.Read()) {
							Console.WriteLine(string.Concat(reader.GetString(0), " - ", reader.GetString(1), " - ", reader.GetString(2), " - ", reader.GetString(3)));
						}

					}  // Il data reader viene chiuso automaticamente alla fine del blocco Using
				} // Il comando viene chiuso automaticamente alla fine del blocco Using
			} // La connessione viene chiusa automaticamente alla fine del blocco Using
		}

		private static void ExecuteScalar() {
			// Creazione della connessione
			using (SqlConnection conn = new SqlConnection(ConfigurationManager.ConnectionStrings["SqlNorthwind"].ConnectionString)) {
				// Apertura della connessione
				conn.Open();

				// Creazione del comando
				using (SqlCommand cmd = new SqlCommand("SELECT COUNT(*) FROM Customers", conn)) {
					// Ritorna un valore intero
					int count = (int)cmd.ExecuteScalar();

					Console.WriteLine(count.ToString());

				} // Il comando viene chiuso automaticamente alla fine del blocco Using

			} // La connessione viene chiusa automaticamente alla fine del blocco Using
		}

		private static void ExecuteNonQuery() {
			// Creazione della connessione
			using (SqlConnection conn = new SqlConnection(ConfigurationManager.ConnectionStrings["SqlNorthwind"].ConnectionString)) {
				// Apertura della connessione
				conn.Open();

				// Creazione del comando
				using (SqlCommand cmd = new SqlCommand("UPDATE Customers SET City='Bergamo' WHERE CustomerID=@CustomerID", conn)) {
					// Aggiungi un parametro
					cmd.Parameters.AddWithValue("@CustomerID", "MAGAA");

					// Ritorna il numero di righe interessate dal comando di aggiornamento
					int rowsAffected = cmd.ExecuteNonQuery();

					Console.WriteLine(rowsAffected.ToString());

				} // Il comando viene chiuso automaticamente alla fine del blocco Using

			} // La connessione viene chiusa automaticamente alla fine del blocco Using
		}
	}
}
