﻿using System.Configuration;
using System.Diagnostics;
using System.IO;
using System.ServiceProcess;
using System.Text;

namespace Aspitalia.Books.Chapter16
{
    public partial class DirectoryWatcher : ServiceBase
    {
        protected string Path { get; set; }
        protected string Filter  { get; set; }
        protected bool IsRecursive  { get; set; }

        public DirectoryWatcher()
        {
            this.Path = @"C:\Temp\";
            this.Filter = "*.*";
            this.IsRecursive = false;

            this.InitializeComponent();
        }

        protected override void OnStart(string[] args)
        {
            this.GetConfiguration();

            this.FileWatcher.Path = this.Path;
            this.FileWatcher.Filter = this.Filter;
            this.FileWatcher.IncludeSubdirectories = this.IsRecursive;

            this.FileWatcher.NotifyFilter = NotifyFilters.LastAccess | NotifyFilters.LastWrite | NotifyFilters.FileName | NotifyFilters.DirectoryName;
            this.FileWatcher.EnableRaisingEvents = true;
        }

        protected override void OnStop()
        {
            this.FileWatcher.EnableRaisingEvents = false;
        }

        protected override void OnPause()
        {
            this.FileWatcher.EnableRaisingEvents = false;
        }

        protected override void OnContinue()
        {
            this.FileWatcher.NotifyFilter = NotifyFilters.LastAccess | NotifyFilters.LastWrite | NotifyFilters.FileName | NotifyFilters.DirectoryName;
            this.FileWatcher.EnableRaisingEvents = true;
        }

        protected override void OnShutdown()
        {
            this.FileWatcher.EnableRaisingEvents = false;
        }

        private void OnFileChanged(object sender, FileSystemEventArgs e)
        {
        StringBuilder builder = new StringBuilder();
        builder.Append(string.Concat("Base directory: '", this.Path, "'.\n"));
        builder.Append(string.Concat("Path: '", e.FullPath, "'.\n"));
        builder.Append(string.Concat("Activity: item '", e.Name, "' changed."));

        string message = builder.ToString();
        EventLog.WriteEntry(this.ServiceName, message);
        }

        private void OnFileCreated(object sender, FileSystemEventArgs e)
        {
            StringBuilder builder = new StringBuilder();
            builder.Append(string.Concat("Base directory: '", this.Path, "'.\n"));
            builder.Append(string.Concat("Path: '", e.FullPath, "'.\n"));
            builder.Append(string.Concat("Activity: item '", e.Name, "' created."));

            string message = builder.ToString();
            EventLog.WriteEntry(this.ServiceName, message);
        }

        private void OnFileDeleted(object sender, FileSystemEventArgs e)
        {
            StringBuilder builder = new StringBuilder();
            builder.Append(string.Concat("Base directory: '", this.Path, "'.\n"));
            builder.Append(string.Concat("Path: '", e.FullPath, "'.\n"));
            builder.Append(string.Concat("Activity: item '", e.Name, "' deleted."));

            string message = builder.ToString();
            EventLog.WriteEntry(this.ServiceName, message);

        }

        private void OnFileRenamed(object sender, RenamedEventArgs e)
        {
            StringBuilder builder = new StringBuilder();
            builder.Append(string.Concat("Base directory: '", this.Path, "'.\n"));
            builder.Append(string.Concat("Path: '", e.OldFullPath, "'.\n"));
            builder.Append(string.Concat("Activity: item '", e.OldName, "' renamed.\n"));
            builder.Append(string.Concat("New name: '", e.Name, "'."));

            string message = builder.ToString();
            EventLog.WriteEntry(this.ServiceName, message);
        }

        private void GetConfiguration()
        {
            if (ConfigurationManager.AppSettings["path"] != null)
                this.Path = ConfigurationManager.AppSettings["path"];

            if (ConfigurationManager.AppSettings["filter"] != null)
                this.Filter = ConfigurationManager.AppSettings["filter"];

            if (ConfigurationManager.AppSettings["isRecursive"] != null)
            {
                bool recursive = false;
                if (bool.TryParse(ConfigurationManager.AppSettings["isRecursive"], out recursive))
                    this.IsRecursive = recursive;
            }
        }
    }
}